/*
 * This file is part of Firestorm NIDS
 * Copyright (c) 2002 Gianni Tedesco
 *
 * 	This is the central registry of all matchers. Usually plugins
 * 	containing realms call us when rules are commited to them.
 */
#include <rlim.h>
#include <stdio.h>
#include <string.h>
#include <packet.h>
#include <matcher.h>
#include <cleanup.h>

struct matcher *matchers=NULL;
unsigned int num_matchers=0;
unsigned int match_warn=0;

/* Always differ */
int null_cmp(void *p1, void *p2)
{
	return 0;
}

void null_cleanup(void *foo){}

struct matcher *matcher_find(char *name)
{
	struct matcher *m;

	for(m=matchers; m; m=m->next) {
		if ( !strcmp(m->name, name) ) {
			return m;
		}
	}
	
	return NULL;
}

/* We get passed an array of matcher structures, terminated
 * by a NULL element. Add them all to the list */
int matcher_add(char *name,
	proc_match_validate v,
	proc_match_compare cmp,
	proc_match_cleanup cl)
{
	struct matcher *m;
	
	if ( !name || !v ) return 0;
	
	if ( !(m=malloc(sizeof(*m))) ) {
		cperror("malloc");
	}

	m->name=name;
	m->validate=v;
	m->compare=cmp ? cmp : null_cmp;	
	m->cleanup=cl ? cl : null_cleanup;
	m->next=matchers;
	matchers=m;
	num_matchers++;

	return 1;
}
