/*
* This file is part of Firestorm NIDS
* Copyright (c) 2002 Gianni Tedesco
*/
#include <string.h>
#include <stdlib.h>
#include <cleanup.h>
#include <template.h>

/* Helper functions for ranges */
static inline int strtoint(char *s, unsigned int *v)
{
	unsigned int d;
	int p;

	for(*v=0,p=0; (d=(unsigned int)(s[p] - '0')) < 10; p++) {
		*v=*v*10+d;
	}

	return p;
}

int p_range(char *arg,
	unsigned int *min,
	unsigned int *max,
	unsigned int mask)
{
	char *hi, *lo;

	if ( (hi=strchr((lo=arg), ':')) ) {
		hi++;
	}else{
		hi=lo;
	}
	
	if ( *lo && *lo!=':' ) {
		if ( !strtoint(lo, min) ) return 0;
		if ( *min & ~mask ) return 0;
	}else{
		*min=0;
	}
	
	if ( *hi ) {
		if ( !strtoint(hi, max) ) return 0;
		if ( *max & ~mask ) return 0;
	}else {
		*max=-1;
		*max&=mask;
	}

	return 1;
}

/* V A L I D A T E  F U N C T I O N S */
int template_longrange(char *args, void **priv)
{
	struct longrange *ret;

	if ( !(ret=malloc(sizeof(*ret))) ) {
		cperror("malloc");
	}

	if ( !p_range(args, &ret->min, &ret->max, -1) ){
		free(ret);
		return 0;
	}

	*priv=ret;
	
	return 1;
}

int template_shortrange(char *args, void **priv)
{
	struct longrange tmp;
	struct shortrange *ret;

	if ( !p_range(args, &tmp.min, &tmp.max, 0xffff) ){
		return 0;
	}

	if ( !(ret=malloc(sizeof(*ret))) ) {
		cperror("malloc");
	}

	ret->min=tmp.min;
	ret->max=tmp.max;

	*priv=ret;
	
	return 1;
}

/* C L E A N U P  F U N C T I O N S */

/* Cleanup a ptr to something */
void template_ptr(void *p){
	if ( p ) free(p);
}

/* Cleanup a pointer to a pointer */
void template_ptrptr(struct ptrptr *p) {
	if ( p ) {
		if ( p->ptr ) free(p->ptr);
		free(p);
	}
}

